package triangle

import "../../tlr"
import "../../tlr/gpu"
import "core:log"

Vertex :: struct{
	position: [2]f32,
}

vertices := []Vertex{
	{{ 0, -0.5 }},
	{{ -0.5, 0.5 }},
	{{ 0.5, 0.5 }},
}

main :: proc() {
	context.logger = log.create_console_logger()
	defer log.destroy_console_logger(context.logger)

	when gpu.BACKEND == .Vulkan {
		gpu.vk_set_hints(gpu.Vk_Hints{
			use_validation_layers = true,
		})
	}

	tlr.init("Triangle Example", 500, 500)
	gpu.init()
	defer {
		gpu.cleanup()
		tlr.cleanup()
	}

	vert_shader, vert_ok := gpu.make_shader("./examples/triangle/triangle.vert.spv")
	frag_shader, frag_ok := gpu.make_shader("./examples/triangle/triangle.frag.spv")
	assert(vert_ok && frag_ok)

	vert_binding := gpu.Vertex_Binding{
		binding = 0,
		stride = size_of(Vertex),
		attributes = {
			{
				location = 0,
				format = .Vec2_Float32,
				offset = offset_of(Vertex, position),
			},
		},
	}
	pipeline, _ := gpu.make_pipeline(gpu.Pipeline_Info{
		vertex = vert_shader,
		fragment = frag_shader,
		vertex_bindings = { vert_binding, },
	})

	vbo, _ := gpu.make_buffer(gpu.Buffer_Info{
		size = size_of(Vertex)*len(vertices),
	})
	gpu.upload_buffer(vbo, vertices)

	for !tlr.window_should_close() {
		gpu.begin_pass(gpu.Pass_Info{
			clear_color = [4]f32{ 1, 0.75, 0, 1 },
		})
		gpu.bind_pipeline(pipeline)
		gpu.bind_vertex_buffer(vbo)
		gpu.draw_elems(len(vertices))
		gpu.end_pass()
	}
}
